"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = require("fs");

var _child_process = require("child_process");

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _dist = require("./dist");

var _testExclude = _interopRequireDefault(require("test-exclude"));

var _schema = _interopRequireDefault(require("@istanbuljs/schema"));

var _parseDiff = _interopRequireDefault(require("./parse-diff"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const path = require('path');

function getRealpath(n) {
  try {
    return (0, _fs.realpathSync)(n) ||
    /* istanbul ignore next */
    n;
  } catch (e) {
    /* istanbul ignore next */
    return n;
  }
}

const memoize = new Map();
/* istanbul ignore next */

const memosep = path.sep === '/' ? ':' : ';';

function loadNycConfig(cwd, opts) {
  let memokey = cwd;
  const args = [path.resolve(__dirname, 'load-nyc-config-sync.js'), cwd];

  if ('nycrcPath' in opts) {
    args.push(opts.nycrcPath);
    memokey += memosep + opts.nycrcPath;
  }
  /* execFileSync is expensive, avoid it if possible! */


  if (memoize.has(memokey)) {
    return memoize.get(memokey);
  }

  const result = JSON.parse((0, _child_process.execFileSync)(process.execPath, args));
  const error = result['load-nyc-config-sync-error'];

  if (error) {
    throw new Error(error);
  }

  const config = { ..._schema.default.defaults.babelPluginIstanbul,
    cwd,
    ...result
  };
  memoize.set(memokey, config);
  return config;
}

function findConfig(opts) {
  const cwd = getRealpath(opts.cwd || process.env.NYC_CWD ||
  /* istanbul ignore next */
  process.cwd());
  const keys = Object.keys(opts);
  const ignored = Object.keys(opts).filter(s => s === 'nycrcPath' || s === 'cwd');

  if (keys.length > ignored.length) {
    // explicitly configuring options in babel
    // takes precedence.
    return { ..._schema.default.defaults.babelPluginIstanbul,
      cwd,
      ...opts
    };
  }

  if (ignored.length === 0 && process.env.NYC_CONFIG) {
    // defaults were already applied by nyc
    return JSON.parse(process.env.NYC_CONFIG);
  }

  return loadNycConfig(cwd, opts);
}

function makeShouldSkip() {
  let exclude;
  return function shouldSkip(file, nycConfig) {
    if (!exclude || exclude.cwd !== nycConfig.cwd) {
      exclude = new _testExclude.default({
        cwd: nycConfig.cwd,
        include: nycConfig.include,
        exclude: nycConfig.exclude,
        extension: nycConfig.extension,
        // Make sure this is true unless explicitly set to `false`. `undefined` is still `true`.
        excludeNodeModules: nycConfig.excludeNodeModules !== false
      });
    }

    return !exclude.shouldInstrument(file);
  };
}

function isValue(item, isNumber = false, defaultNumber) {
  let data = '';

  if (isNumber) {
    data = item ? Number(item) : defaultNumber;
  } else {
    data = item ? item : defaultNumber;
  }

  return data;
}

var _default = (0, _helperPluginUtils.declare)(api => {
  api.assertVersion(7);
  const shouldSkip = makeShouldSkip();
  const t = api.types;
  return {
    visitor: {
      Program: {
        enter(path) {
          // 筛选后的数据增量文件
          let diffScreen = null;
          this.__dv__ = null;
          this.nycConfig = findConfig(this.opts);
          const realPath = getRealpath(this.file.opts.filename);
          const {
            instrmenttation = {}
          } = this.nycConfig; // npm引入参数

          let IS_PITCHING_PILE_DATA = false;
          let INCREMENT_DATA = false;
          let IS_HTML_CSS = false; // 是否能看html结构
          let BRANCH_DATA = 'origin/master';
          let processData = {};
          Object.keys(process.env).forEach(item => {
            let isItem = process.env[item];

            if (item.indexOf('IS_HTML_CSS') !== -1) {
              processData.IS_HTML_CSS = isItem;
            }

            if (item.indexOf('INCREMENT') !== -1) {
              processData.INCREMENT = isItem;
            }

            if (item.indexOf('IS_PITCHING_PILE') !== -1) {
              processData.IS_PITCHING_PILE = isItem;
            }

            if (item.indexOf('BRANCH') !== -1) {
              processData.BRANCH = process.env[item];
            } // Object.keys(processData).length

          }); // 全局变量优先
          if (Object.keys(instrmenttation).length) {
            BRANCH_DATA = isValue(instrmenttation.BRANCH, false, 'origin/master');
            INCREMENT_DATA = isValue(instrmenttation.INCREMENT, false, 'false');
            IS_PITCHING_PILE_DATA = isValue(instrmenttation.IS_PITCHING_PILE, false, false);
            IS_HTML_CSS = isValue(instrmenttation.IS_HTML_CSS, false, false);
          } else if (Object.keys(processData).length) {
            BRANCH_DATA = isValue(processData.BRANCH, false, 'origin/master');
            INCREMENT_DATA = isValue(processData.INCREMENT, false, 'false');
            IS_PITCHING_PILE_DATA = isValue(processData.IS_PITCHING_PILE, false, false);
            IS_HTML_CSS = isValue(processData.IS_HTML_CSS, false, false);
          } // instrumenttation (branch: git diff 跟那个分支对比, increment 是否开启增量代码检测) 拿配置到底是全量代码还是增量代码


          if (!IS_PITCHING_PILE_DATA || IS_PITCHING_PILE_DATA === 'false') {
            return false;
          }

          const changeList = []; // 下面的是零时的
          // // const gitDiffCode = execSync(`git diff ${branch}`)
          // const diffData = parseDiffData(gitDiffCode.toString())
          // diffScreen = diffData.find(item => {
          //   return realPath.indexOf(item.to) > -1
          // })
          // 后面解开

          if ((INCREMENT_DATA || INCREMENT_DATA === 'true') && BRANCH_DATA) {
            const gitDiffCode = (0, _child_process.execSync)(`git add . && git diff --cached ${BRANCH_DATA}`);
            const diffData = (0, _parseDiff.default)(gitDiffCode.toString());
            const less = '.less';
            const css = '.css';
            diffScreen = diffData.find(item => {
              if (IS_HTML_CSS) {
                return realPath.indexOf(item.to) > -1;
              } else {
                return realPath.indexOf(item.to) > -1 && realPath.indexOf(less) === -1 && realPath.indexOf(css) === -1;
              }
            });
          }

          if (shouldSkip(realPath, this.nycConfig)) {
            return;
          }

          let {
            inputSourceMap
          } = this.opts;

          if (this.opts.useInlineSourceMaps) {
            if (!inputSourceMap && this.file.inputMap) {
              inputSourceMap = this.file.inputMap.sourcemap;
            }
          }

          const visitorOptions = {};
          Object.entries(_schema.default.defaults.instrumentVisitor).forEach(([name, defaultValue]) => {
            if (name in this.nycConfig) {
              visitorOptions[name] = this.nycConfig[name];
            } else {
              visitorOptions[name] = _schema.default.defaults.instrumentVisitor[name];
            }
          }); // 后增加|| !increment这个条件判断条件后面加上

          if (diffScreen !== null && diffScreen && (INCREMENT_DATA || INCREMENT_DATA === 'true')) {
            // if (diffScreen) {
            if (Array.isArray(diffScreen && diffScreen.chunks)) {
              diffScreen.chunks.forEach(item => {
                let changes = Array.isArray(item.changes) ? item.changes : [];
                changes.forEach(items => {
                  if (items && items.add) {
                    changeList.push(items.ln);
                  }
                });
              });
            }
            if (changeList.length) {
              visitorOptions.changeList = changeList;
              visitorOptions.IS_HTML_CSS = IS_HTML_CSS; // changeList

              this.__dv__ = (0, _dist.programVisitor)(t, realPath, { ...visitorOptions,
                inputSourceMap
              });

              this.__dv__.enter(path);
            } // }

          } else if (!INCREMENT_DATA || INCREMENT_DATA === 'false') {
            visitorOptions.changeList = changeList;
            visitorOptions.IS_HTML_CSS = IS_HTML_CSS;
            this.__dv__ = (0, _dist.programVisitor)(t, realPath, { ...visitorOptions,
              inputSourceMap
            });

            this.__dv__.enter(path);
          }
        },

        exit(path) {
          if (!this.__dv__) {
            return;
          }

          const result = this.__dv__.exit(path);

          if (this.opts.onCover) {
            this.opts.onCover(getRealpath(this.file.opts.filename), result.fileCoverage);
          }
        }

      }
    }
  };
}); // export default declare(api => {
//   api.assertVersion(7)
//   console.log(api, 'api')
//   const shouldSkip = makeShouldSkip()
//   const t = api.types
//   return {
//     visitor: {
//       Program: {
//         enter (path) {
//           console.log(121212121)
//           this.__dv__ = null
//           this.nycConfig = findConfig(this.opts)
//           const realPath = getRealpath(this.file.opts.filename)
//           if (shouldSkip(realPath, this.nycConfig)) {
//             return
//           }
//           let { inputSourceMap } = this.opts
//           if (this.opts.useInlineSourceMaps !== false) {
//             if (!inputSourceMap && this.file.inputMap) {
//               inputSourceMap = this.file.inputMap.sourcemap
//             }
//           }
//           const visitorOptions = {}
//           Object.entries(schema.defaults.instrumentVisitor).forEach(([name, defaultValue]) => {
//             if (name in this.nycConfig) {
//               visitorOptions[name] = this.nycConfig[name]
//             } else {
//               visitorOptions[name] = schema.defaults.instrumentVisitor[name]
//             }
//           })
//           this.__dv__ = programVisitor(t, realPath, {
//             ...visitorOptions,
//             inputSourceMap
//           })
//           this.__dv__.enter(path)
//         },
//         exit (path) {
//           if (!this.__dv__) {
//             return
//           }
//           const result = this.__dv__.exit(path)
//           if (this.opts.onCover) {
//             this.opts.onCover(getRealpath(this.file.opts.filename), result.fileCoverage)
//           }
//         }
//       }
//     }
//   }
// })
// 增量代码babel引入方法 babel.js 或 babel.config.js
// 'plugins': [
//   ['@hliang/babel-plugin-transfrom-modules-commonjs', {
//     extension: ['.js', '.vue'],
//     instrmenttation: {
//       increment: true, // 是否全量插桩
//       branch: 'origin/master' // 对比分支
//     }
//   }]
// ]


exports.default = _default;